/*
 * LCD device support for the JZ4740 and related SoCs.
 *
 * Copyright (C) 2018, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/lcd.h>
#include <l4/devices/lcd-device.h>

#include <l4/re/c/dma_space.h>
#include <l4/re/c/video/view.h>
#include <l4/sys/types.h>



/* C++ language interface. */

#ifdef __cplusplus

/* Support for specific JZ4780-based devices. */

class Lcd_jz4780_spi_device : public Lcd_device
{
  /* DMA region providing the DMA descriptor virtual and physical addresses. */

  struct dma_region _desc_region;

protected:
  /* Device-specific memory allocation. */

  int _setup_memory();

public:
  Lcd_jz4780_spi_device(Activation *display)
  : Lcd_device(display)
  {
    _setup_memory();
  }

  /* Device operations. */

  void disable();
  void enable();

  l4_size_t get_framebuffer_size();

  void get_view_info(l4re_video_view_info_t *view_info);

  /* Common device instance. */

  static Lcd_jz4780_spi_device *device;
};

#endif
