/*
 * Generic access to general purpose input/output functionality.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <string.h>
#include "gpio-generic.h"

/* Function declarations reproduced here to avoid clock identifier conflicts in
   the chip-specific headers. */

extern Hw::Gpio_chip *jz4730_gpio_chip(l4_addr_t, uint8_t, bool);
extern Hw::Gpio_chip *jz4740_gpio_chip(l4_addr_t, uint8_t, bool);
extern Hw::Gpio_chip *jz4780_gpio_chip(l4_addr_t, uint8_t, bool);
extern Hw::Gpio_chip *x1600_gpio_chip(l4_addr_t, uint8_t, bool);

struct gpio_function
{
  const char *name;
  Hw::Gpio_chip *(*function)(l4_addr_t, uint8_t, bool);
};

static struct gpio_function functions[] = {
  {"jz4730", jz4730_gpio_chip},
  {"jz4740", jz4740_gpio_chip},
  {"jz4780", jz4780_gpio_chip},
  {"x1600", x1600_gpio_chip},
  {NULL, NULL}
};

Hw::Gpio_chip *new_gpio_chip(const char *name, l4_addr_t gpio_base,
                             uint8_t port_number, bool shadow)
{
  for (struct gpio_function *f = functions; f->name != NULL; f++)
  {
    if (!strcmp(name, f->name))
      return f->function(gpio_base, port_number, shadow);
  }

  return NULL;
}
